## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
$$
"""
__author__  = ''
__docformat__ = 'restructuredtext'

# Python imports
import re
import os
import sys
from StringIO import StringIO
from ConfigParser import RawConfigParser

# Zope imports
from zope.interface import implements
from DateTime import DateTime

# Other imports
from iw.fss.interfaces import IStrategy
from iw.fss.utils import copy_file, move_file, rm_file

class BaseStorageStrategy:
    """Strategy of how field values are stored on filesystem"""
    implements(IStrategy)
    
    def __init__(self, storage_path, backup_path):
        """Initialize strategy

        @param storage_path: Filesystem path where field values are stored
        @param backup_path: Filesystem path where field values are backup
        """

        self.storage_path = storage_path
        self.backup_path = backup_path

    def createSubDirectories(self, path, root_path):
        """Create all sub directories if they don't exist"""

        path = os.path.abspath(path)
        root_path = os.path.abspath(root_path)
        if not path.startswith(root_path):
            raise ValueError, "Path %s should be a part of %s" % (root_path, path)

        if not os.path.exists(root_path):
            raise ValueError, "Path %s should exist" % root_path

        if not os.path.exists(path):
            os.makedirs(path)

    def removeSubDirectories(self, path, root_path):
        """Remove all sub directories if they are empty"""

        path = os.path.abspath(path)
        root_path = os.path.abspath(root_path)
        if not path.startswith(root_path):
            raise ValueError, "Path %s should be a part of %s" % (root_path, path)

        def removeEmptyDirectory(cur_path, root_path):
            if cur_path == root_path:
                return
            if not os.path.exists(cur_path):
                return
            if os.listdir(cur_path):
                return
            os.rmdir(cur_path)
            parent_path = os.path.dirname(cur_path)
            removeEmptyDirectory(parent_path, root_path)

        removeEmptyDirectory(path, root_path)

    def walkOnStorageDirectory(self, **kwargs):
        """Walk on storage directory"""

        raise NotImplementedError('walkOnStorageDirectory')

    def walkOnBackupDirectory(self, **kwargs):
        """Walk on backup directory"""

        raise NotImplementedError('walkOnBackupDirectory')

    def walkOnValueDirectoryPath(self, **kwargs):
        """Get path of directory where the field value is stored"""

        raise NotImplementedError('getValueDirectoryPath')

    def getValueDirectoryPath(self, **kwargs):
        """Get path of directory where the field value is stored"""

        raise NotImplementedError('getValueDirectoryPath')

    def getValueFilename(self, **kwargs):
        """Get filename of the field value on filesystem"""

        raise NotImplementedError('getValueFilename')

    def getValueFilePath(self, **kwargs):
        """Get path of file where the field value is stored"""

        raise NotImplementedError('getValueFilePath')

    def getRDFDirectoryPath(self, **kwargs):
        """Get path of directory where the rdf value is stored"""

        raise NotImplementedError('getRDFDirectoryPath')

    def getRDFFilename(self, **kwargs):
        """Get filename of the rdf value on filesystem"""

        raise NotImplementedError('getRDFFilename')

    def getRDFFilePath(self, **kwargs):
        """Get path of file where the rdf value is stored"""

        raise NotImplementedError('getRDFFilePath')

    def getBackupDirectoryPath(self, **kwargs):
        """Get path of directory where the file value is backup"""

        raise NotImplementedError('getBackupDirectoryPath')

    def getBackupFilename(self, **kwargs):
        """Get filename of the file backup value on filesystem"""

        raise NotImplementedError('getBackupFilename')

    def getBackupFilePath(self, **kwargs):
        """Get path of file where the file value is backup"""

        raise NotImplementedError('getBackupFilePath')

    def setValueFile(self, value, **kwargs):
        """Copy file value on filesystem"""

        raise NotImplementedError('setValueFile')

    def unsetValueFile(self, **kwargs):
        """Remove file value if exists"""

        raise NotImplementedError('unsetValueFile')

    def moveValueFile(self, **kwargs):
        """File properties has changed, move it its new locations"""

        raise NotImplementedError('moveValueFile')

    def restoreValueFile(self, **kwargs):
        """Restore the backup value if exists"""

        raise NotImplementedError('restoreValueFile')

    def copyValueFile(self, **kwargs):
        """Duplicate file value on filesystem"""

        raise NotImplementedError('copyValueFile')

class FlatStorageStrategy(BaseStorageStrategy):
    """All field values are stored in a flat structure

    Filename of these stored values: <uid of content>_<field name>
    Filename of rdf file: <uid of content>_<field name>.rdf
    Filename of backup values: <uid of content>_<field name>.bak
    Rdf files are not backup. They are automatically generated

    Example of storage:
    fssDirectory
    |
    |- f42ad00adb7d4580f929d734bf1ed3ab_image
    |
    |- f42ad00adb7d4580f929d734bf1ed3ab_image.rdf
    |
    |- f42ad00adb7d4580f929d734bf1ed3ab_file
    |
    |- f42ad00adb7d4580f929d734bf1ed3ab_file.rdf

    fssBackupDirectory
    |
    |- 9efeb7638fb35f5f9b9955da3e2dbfec_file.bak
    """

    def getValueDirectoryPath(self, **kwargs):
        """Get path of directory where the field value is stored"""

        return self.storage_path

    def getValueFilename(self, **kwargs):
        """Get filename of the field value on filesystem"""

        if kwargs.get('version', None) is not None:
            return '%s_%s.%s' % (kwargs['uid'], kwargs['name'],
                                 kwargs['version'])

        return '%s_%s' % (kwargs['uid'], kwargs['name'])

    def getValueFilePath(self, **kwargs):
        """Get path of file where the field value is stored"""

        return os.path.join(self.getValueDirectoryPath(**kwargs), self.getValueFilename(**kwargs))

    def getRDFDirectoryPath(self, **kwargs):
        """Get path of directory where the rdf value is stored"""

        return self.getValueDirectoryPath(**kwargs)

    def getRDFFilename(self, **kwargs):
        """Get filename of the rdf value on filesystem"""

        if kwargs.get('version', None) is not None:
            return '%s_%s.%s.rdf' % (kwargs['uid'], kwargs['name'],
                                     kwargs['version'])

        return '%s_%s.rdf' % (kwargs['uid'], kwargs['name'])

    def getRDFFilePath(self, **kwargs):
        """Get path of file where the rdf value is stored"""

        return os.path.join(self.getRDFDirectoryPath(**kwargs), self.getRDFFilename(**kwargs))

    def getBackupDirectoryPath(self, **kwargs):
        """Get path of directory where the file value is backup"""

        return self.backup_path

    def getBackupFilename(self, **kwargs):
        """Get filename of the file backup value on filesystem"""

        if kwargs.get('version', None) is not None:
            return '%s_%s.%s.bak' % (kwargs['uid'], kwargs['name'],
                                     kwargs['version'])

        return '%s_%s.bak' % (kwargs['uid'], kwargs['name'])

    def getBackupFilePath(self, **kwargs):
        """Get path of file where the file value is backup"""

        return os.path.join(self.getBackupDirectoryPath(**kwargs), self.getBackupFilename(**kwargs))

    def walkOnUIDDirectory(self, path, reg, **kwargs):
        """Return a list of dictionnary.

        Each dictionnary has this structure:
        - uid -> uid of linked content
        - name -> name of field
        - modified -> DateTime of last modification
        - path -> File path
        """

        items = []

        node_ids = os.listdir(path)
        for node_id in node_ids:
            match = reg.match(node_id)
            if match is None:
                continue

            uid = match.group('uid')
            name = match.group('name')
            file_path = os.path.join(path, node_id)
            modified = os.path.getmtime(file_path)
            size = os.path.getsize(file_path)

            # Store item
            item = {}
            item['uid'] = uid
            item['name'] = name
            item['fs_path'] = file_path
            item['modified'] = DateTime(modified)
            item['size'] = size
            items.append(item)

        return items

    def walkOnStorageDirectory(self, **kwargs):
        """Walk on storage directory"""

        reg = re.compile(r'^(?P<uid>.{32})_(?P<name>[^.]*)$')
        return self.walkOnUIDDirectory(self.storage_path, reg)

    def walkOnBackupDirectory(self, **kwargs):
        """Walk on backup directory"""

        reg = re.compile(r'^(?P<uid>.{32})_(?P<name>[^.]*).bak$')
        return self.walkOnUIDDirectory(self.backup_path, reg)

    def setValueFile(self, value, **kwargs):
        """Copy file value on filesystem"""

        # Put value in buffer
        buffer = StringIO(value)
        copy_file(buffer, self.getValueFilePath(**kwargs))
        buffer.close()

    def unsetValueFile(self, **kwargs):
        """Remove file value if exists"""

        # This is a cut/paste operation. There is no need to delete file
        if kwargs.get('is_moved', False):
            return

        value_path = self.getValueFilePath(**kwargs)
        if not os.path.exists(value_path):
            return
        backup_path = self.getBackupFilePath(**kwargs)
        move_file(value_path, backup_path)

    def restoreValueFile(self, **kwargs):
        """Restore the backup value if exists"""

        backup_path = self.getBackupFilePath(**kwargs)
        if not os.path.exists(backup_path):
            return
        value_path = self.getValueFilePath(**kwargs)
        move_file(backup_path, value_path)

    def moveValueFile(self, **kwargs):
        """File properties has changed, move it its new locations"""

        # Delete uid to avoid multiple key errors
        new_kwargs = kwargs.copy()
        del new_kwargs['uid']

        src_path = self.getValueFilePath(uid=kwargs['src_uid'], **new_kwargs)
        if not os.path.exists(src_path):
            return
        dst_path = self.getValueFilePath(**kwargs)
        move_file(src_path, dst_path)

    def copyValueFile(self, **kwargs):
        """Duplicate file value on filesystem"""

        src_path = self.getValueFilePath(uid=kwargs['src_uid'], name=kwargs['name'])
        if not os.path.exists(src_path):
            return
        dst_path = self.getValueFilePath(**kwargs)
        copy_file(src_path, dst_path)

    def setRDFFile(self, value, **kwargs):
        """Copy rdf file on filesystem"""

        # Put value in buffer
        buffer = StringIO(value)
        copy_file(buffer, self.getRDFFilePath(**kwargs))
        buffer.close()

    def unsetRDFFile(self, **kwargs):
        """Remove rdf file if exists"""

        # This is a cut/paste operation. There is no need to delete file
        if kwargs.get('is_moved', False):
            return

        rdf_path = self.getRDFFilePath(**kwargs)
        if not os.path.exists(rdf_path):
            return
        rm_file(rdf_path)

class DirectoryStorageStrategy(FlatStorageStrategy):
    """All field values are stored in a directory structure.
    Sub directories are defined on two level.
    First level of directory uses the 2 characters of content uid.
    Second level of directory uses the 4 characters of content uid.
    Backup files are stored in a flat structure.

    Filename of these stored values: <uid of content>_<field name>
    Filename of rdf file: <uid of content>_<field name>.rdf
    Filename of backup values: <uid of content>_<field name>.bak
    Rdf files are not backup. They are automatically generated

    Example of storage:
    fssDirectory
    |
    |- f42
       |
       |- f42ad
          |
          |- f42ad00adb7d4580f929d734bf1ed3ab_image
          |
          |- f42ad00adb7d4580f929d734bf1ed3ab_image.rdf
          |
          |- f42ad00adb7d4580f929d734bf1ed3ab_file
          |
          |- f42ad00adb7d4580f929d734bf1ed3ab_file.rdf

    fssBackupDirectory
    |
    |- 9efeb7638fb35f5f9b9955da3e2dbfec_file.bak
    """

    def walkOnStorageDirectory(self, **kwargs):
        """Walk on storage directory"""

        reg = re.compile(r'^(?P<uid>.{32})_(?P<name>[^.]*)$')

        # Get all paths to walk on
        paths = []
        for root, dirs, files in os.walk(self.storage_path):
            if dirs:
                # We have reached last level
                continue
            paths.append(root)

        # Walk on each path to get all information
        items = []
        for path in paths:
            items.extend(self.walkOnUIDDirectory(path, reg))
        return items

    def getValueDirectoryPath(self, **kwargs):
        """Get path of directory where the field value is stored"""

        uid = kwargs['uid']
        first_level = uid[:2]
        second_level = uid[:4]
        return os.path.join(self.storage_path, first_level, second_level)

    def setValueFile(self, value, **kwargs):
        """Copy file value on filesystem"""

        self.createSubDirectories(self.getValueDirectoryPath(**kwargs), self.storage_path)
        FlatStorageStrategy.setValueFile(self, value, **kwargs)

    def unsetValueFile(self, **kwargs):
        """Remove file value if exists"""

        FlatStorageStrategy.unsetValueFile(self, **kwargs)
        self.removeSubDirectories(self.getValueDirectoryPath(**kwargs), self.storage_path)

    def restoreValueFile(self, **kwargs):
        """Restore the backup value if exists"""

        self.createSubDirectories(self.getValueDirectoryPath(**kwargs), self.storage_path)
        FlatStorageStrategy.restoreValueFile(self, **kwargs)

    def moveValueFile(self, **kwargs):
        """File properties has changed, move it to its new location"""

        # Delete uid to avoid multiple key errors
        new_kwargs = kwargs.copy()
        del new_kwargs['uid']

        # Move File
        self.createSubDirectories(self.getValueDirectoryPath(uid=kwargs['uid'], **new_kwargs), self.storage_path)
        FlatStorageStrategy.moveValueFile(self, **kwargs)

        # Delete old directories
        self.removeSubDirectories(self.getValueDirectoryPath(uid=kwargs['src_uid'], **new_kwargs), self.storage_path)

    def copyValueFile(self, **kwargs):
        """Duplicate file value on filesystem"""

        self.createSubDirectories(self.getValueDirectoryPath(**kwargs), self.storage_path)
        FlatStorageStrategy.copyValueFile(self, **kwargs)

    def setRDFFile(self, value, **kwargs):
        """Copy rdf file on filesystem"""

        self.createSubDirectories(self.getRDFDirectoryPath(**kwargs), self.storage_path)
        FlatStorageStrategy.setRDFFile(self, value, **kwargs)

    def unsetRDFFile(self, **kwargs):
        """Remove rdf file if exists"""

        FlatStorageStrategy.unsetRDFFile(self, **kwargs)
        self.removeSubDirectories(self.getRDFDirectoryPath(**kwargs), self.storage_path)

class SiteStorageStrategy(DirectoryStorageStrategy):
    """All field values are stored in a directory structure mirroring structure
    of PloneSite.
    Backup files are stored in a flat structure.

    Filename of these stored values: Filename of field value or field name
    if not defined
    Filename of rdf file: <field name>.rdf
    Filename of backup values: <uid of content>_<field name>.bak
    Rdf files are not backup. They are automatically generated

    Example of storage:
    fssDirectory
    |
    |- members
       |
       |- john
          |
          |- dummy-document
             |
             |- image
             |  |
             |  |- moutain.jpg
             |  |
             |  |- image.rdf
             |
             |- file
                |
                |- diary.odt
                |
                |- file.rdf

    fssBackupDirectory
    |
    |- 9efeb7638fb35f5f9b9955da3e2dbfec_file.bak
    """

    def walkOnStorageDirectory(self, **kwargs):
        """Walk on storage directory"""

        # Get all paths to walk on
        paths = []
        root_path = os.path.abspath(self.storage_path)
        for root, dirs, files in os.walk(root_path):
            if dirs:
                # We have reached last level
                continue
            paths.append(root)

        items = []

        for path in paths:
            name = os.path.basename(path)
            obj_path = os.path.dirname(path)[(len(root_path) + 1):]
            if sys.platform == 'win32':
                obj_path = obj_path.replace('\\', '/')
            node_ids = os.listdir(path)
            rdf_filename = self.getRDFFilename(name=name)

            # There is only one field value for a path
            for node_id in node_ids:
                if node_id == rdf_filename:
                    continue
                file_path = os.path.join(path, node_id)
                modified = os.path.getmtime(file_path)
                size = os.path.getsize(file_path)

                # Store item
                item = {}
                item['path'] = obj_path
                item['name'] = name
                item['fs_path'] = file_path
                item['modified'] = DateTime(modified)
                item['size'] = size
                items.append(item)

        return items

    def getValueFilename(self, **kwargs):
        """Get filename of the field value on filesystem"""

        filename = kwargs.get('title', '')

        if not filename:
            filename = kwargs['name']

        if kwargs.get('version', None) is not None:
            filename = '%s.%s' % (filename, kwargs['version'])

        # Maybe filename of field is the same than rdf filename
        rdf_filename = self.getRDFFilename(**kwargs)

        if rdf_filename == filename:
            filename = '-%s' % filename

        return filename

    def getValueDirectoryPath(self, **kwargs):
        """Get path of directory where the field value is stored"""

        path_list = []
        path_list.append(self.storage_path)
        path_list.extend(kwargs['path'].split('/'))
        path_list.append(kwargs['name'])
        return os.path.join(*path_list)

    def getRDFFilename(self, **kwargs):
        """Get filename of the rdf value on filesystem"""

        if kwargs.get('version', None) is not None:
            return '%s.%s.rdf' % (kwargs['name'], kwargs['version'])

        return '%s.rdf' % kwargs['name']

    def copyValueFile(self, **kwargs):
        """Duplicate file value on filesystem"""

        src_path = self.getValueFilePath(uid=kwargs['src_uid'], path=kwargs['src_path'], name=kwargs['name'], title=kwargs.get('title', ''))

        if not os.path.exists(src_path):
            return

        dst_path = self.getValueFilePath(**kwargs)
        self.createSubDirectories(self.getValueDirectoryPath(**kwargs), self.storage_path)
        copy_file(src_path, dst_path)

    def setValueFile(self, value, **kwargs):
        """Copy file value on filesystem"""

        # Remove all files in value directory except rdf file
        path = self.getValueDirectoryPath(**kwargs)

        if os.path.exists(path):
            rdf_filename = self.getRDFFilename(**kwargs)
            for filename in os.listdir(path):
                if filename == rdf_filename:
                    continue
                file_path = os.path.join(path, filename)
                rm_file(file_path)

        DirectoryStorageStrategy.setValueFile(self, value, **kwargs)

    def unsetValueFile(self, **kwargs):
        """Remove file value if exists"""

        # Always delete old path when moving objects in this strategy
        if kwargs.has_key('is_moved'):
            del kwargs['is_moved']
        DirectoryStorageStrategy.unsetValueFile(self, **kwargs)

    def unsetRDFFile(self, **kwargs):
        """Remove rdf file if exists"""

        # Always delete old path when moving objects in this strategy
        if kwargs.has_key('is_moved'):
            del kwargs['is_moved']
        DirectoryStorageStrategy.unsetRDFFile(self, is_deleted=True, **kwargs)

class SiteStorageStrategy2(DirectoryStorageStrategy):
    """All field values are stored in a directory structure mirroring structure
    of PloneSite.
    Backup files are stored in a flat structure.

    Filename of these stored values: Filename of field value or field name
    if not defined
    Filename of rdf file: <field filename>.rdf
    Filename of backup values: <uid of content>_<field name>.bak
    Rdf files are not backup. They are automatically generated

    Example of storage:
    fssDirectory
    |
    |- members
       |
       |- john
          |
          |- dummy-document
             |
             |- fss.cfg
             |
             |- moutain.jpg
             |
             |- mountain.jpg.rdf
             |
             |- diary.odt
             |
             |- diary.odt.rdf

    fssBackupDirectory
    |
    |- 9efeb7638fb35f5f9b9955da3e2dbfec_file.bak
    """

    cfg_filename = "fss.cfg"
    cfg_filename_section = "FILENAME"

    def walkOnStorageDirectory(self, **kwargs):
        """Walk on storage directory"""

        # Get all paths to walk on
        paths = []
        root_path = os.path.abspath(self.storage_path)
        for root, dirs, files in os.walk(root_path):
            if dirs:
                # We have reached last level
                continue
            paths.append(root)

        items = []

        for path in paths:
            # Get config files
            cfg_path = os.path.join(path, self.cfg_filename)

            if not os.path.exists(cfg_path):
                continue

            # Update file
            config = RawConfigParser()

            # Read file
            fd = open(cfg_path, 'r')
            try:
                config.readfp(fd)
            finally:
                fd.close()

            obj_path = path[(len(root_path) + 1):]
            if sys.platform == 'win32':
                obj_path = obj_path.replace('\\', '/')

            names = config.options(self.cfg_filename_section)

            # Get filenames
            for name in names:
                filename = config.get(self.cfg_filename_section, name)
                file_path = os.path.join(path, filename)
                modified = os.path.getmtime(file_path)
                size = os.path.getsize(file_path)

                # Store item
                item = {}
                item['path'] = obj_path
                item['name'] = name
                item['fs_path'] = file_path
                item['modified'] = DateTime(modified)
                item['size'] = size
                items.append(item)

        return items

    def getFilenameInConfigFile(self, root_path, name):
        """Get filename field association in config file

        @root_path: Path where config file is stored
        @param name: Field name
        """

        path = os.path.join(root_path, self.cfg_filename)

        # Config file doesn't exists
        if not os.path.exists(path):
            return None

        # If it exists parse it and get value
        fd = open(path, 'r')
        value = None
        try:
            config = RawConfigParser()
            config.readfp(fd)
            if config.has_section(self.cfg_filename_section) and \
               config.has_option(self.cfg_filename_section, name):
                value = config.get(self.cfg_filename_section, name)
        finally:
            fd.close()

        return value

    def setFilenameInConfigFile(self, root_path, name, filename):
        """Add new filename association in config file

        @root_path: Path where config file is stored
        @param name: Field name
        @param filename: Filename of value stored in field
        """

        path = os.path.join(root_path, self.cfg_filename)

        # Update file
        config = RawConfigParser()

        if os.path.exists(path):
            # Read file
            fd = open(path, 'r')
            try:
                config.readfp(fd)
            finally:
                fd.close()

        # Create section if it doesn't exist
        if not config.has_section(self.cfg_filename_section):
            config.add_section(self.cfg_filename_section)
        config.set(self.cfg_filename_section, name, filename)

        fd = open(path, 'w')
        try:
            # Write file
            config.write(fd)
        finally:
            fd.close()

    def unsetFilenameInConfigFile(self, root_path, name):
        """Remove filename field association in config file

        @root_path: Path where config file is stored
        @param name: Field name
        """

        path = os.path.join(root_path, self.cfg_filename)

        # Config file doesn't exists
        if not os.path.exists(path):
            return

        # Initialize config file
        config = RawConfigParser()

        # Read old config file
        fd = open(path, 'r')
        try:
            # Read file
            config.readfp(fd)
        finally:
            fd.close()

        # Update config file
        if config.has_section(self.cfg_filename_section):
            config.remove_option(self.cfg_filename_section, name)

        if not config.options(self.cfg_filename_section):
            # Section is empty so remove config file
            rm_file(path)
        else:
            fd = open(path, 'w')
            try:
                # Write file
                config.write(fd)
            finally:
                fd.close()

    def getValueFilename(self, **kwargs):
        """Get filename of the field value on filesystem"""

        path = self.getValueDirectoryPath(**kwargs)
        name = kwargs['name']

        # Maybe filename has already be defined
        if os.path.exists(path):
            filename = self.getFilenameInConfigFile(path, name)

            if filename is not None:
                return filename

        # Filename has not been already defined
        filename = kwargs.get('title', '')

        if not filename:
            filename = name

        if kwargs.get('version', None) is not None:
            filename = '%s.%s' % (filename, kwargs['version'])

        # Filename is unique because path has not been created
        if not os.path.exists(path):
            return filename

        # Make sure filename doesn't exist and is not using a reserved name
        # reserved name is defined in cfg_filename variable
        node_ids = os.listdir(path)

        # Generate filename
        orig_filename = filename
        filename_created = False
        index = 0
        while not filename_created:
            if filename not in node_ids:
                filename_created = True
            else:
                index += 1
                filename = '%d-%s' % (index, filename)

        return filename

    def getValueDirectoryPath(self, **kwargs):
        """Get path of directory where the field value is stored"""

        path_list = []
        path_list.append(self.storage_path)
        path_list.extend(kwargs['path'].split('/'))
        return os.path.join(*path_list)

    def getRDFFilename(self, **kwargs):
        """Get filename of the rdf value on filesystem"""

        if kwargs.get('version', None) is not None:
            return '%s.%s.rdf' % (self.getValueFilename(**kwargs),
                                  kwargs['version'])

        return '%s.rdf' % self.getValueFilename(**kwargs)

    def setValueFile(self, value, **kwargs):
        """Copy file value on filesystem"""

        # Get filename of value file
        name = kwargs['name']
        path = self.getValueDirectoryPath(**kwargs)

        # Remove old files (value and rdf files)
        file_path = self.getValueFilePath(**kwargs)
        if os.path.exists(file_path):
            rm_file(file_path)
        rdf_path = self.getRDFFilePath(**kwargs)
        if os.path.exists(rdf_path):
            rm_file(rdf_path)
        self.unsetFilenameInConfigFile(path, name)

        # Process new filename
        filename = self.getValueFilename(**kwargs)

        # Update config file
        self.createSubDirectories(path, self.storage_path)
        self.setFilenameInConfigFile(path, name, filename)

        # Store value
        FlatStorageStrategy.setValueFile(self, value, **kwargs)

    def unsetValueFile(self, **kwargs):
        """Remove file value if exists"""

        name = kwargs['name']
        path = self.getValueDirectoryPath(**kwargs)

        # Always delete old path when moving objects in this strategy
        if kwargs.has_key('is_moved'):
            del kwargs['is_moved']

        # Remove file
        FlatStorageStrategy.unsetValueFile(self, **kwargs)
        self.unsetRDFFile(**kwargs)

        # Remove entry in config file
        self.unsetFilenameInConfigFile(path, name)

        # Remove subdirectory if empty
        self.removeSubDirectories(path, self.storage_path)

    def restoreValueFile(self, **kwargs):
        """Restore the backup value if exists"""

        name = kwargs['name']
        path = self.getValueDirectoryPath(**kwargs)

        # Process new filename
        filename = self.getValueFilename(**kwargs)

        # Update config file
        self.createSubDirectories(path, self.storage_path)
        self.setFilenameInConfigFile(path, name, filename)

        # Restore value
        FlatStorageStrategy.restoreValueFile(self, **kwargs)

    def copyValueFile(self, **kwargs):
        """Duplicate file value on filesystem"""

        src_path = self.getValueFilePath(uid=kwargs['src_uid'], path=kwargs['src_path'], name=kwargs['name'], title=kwargs.get('title', ''))

        if not os.path.exists(src_path):
            return

        name = kwargs['name']
        path = self.getValueDirectoryPath(**kwargs)

        # Process new filename
        filename = self.getValueFilename(**kwargs)

        # Update config file
        self.createSubDirectories(path, self.storage_path)
        self.setFilenameInConfigFile(path, name, filename)

        # Then copy file
        dst_path = self.getValueFilePath(**kwargs)
        copy_file(src_path, dst_path)
        return




## -*- coding: utf-8 -*-
## Copyright (C) 2008 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# $Id: test_zcml.py 65108 2008-05-19 20:23:31Z cbosse $
"""
Testing the ZCML meta and associated modules
"""
__author__  = ''
__docformat__ = 'restructuredtext'

import unittest
from zope.testing import doctest
from zope.testing.doctest import ELLIPSIS

def test_typewithfss():
    """
    Test fss:typeWithFSS directive::

        >>> from Products.Five import zcml
        >>> import iw.fss
        >>> template = '''
        ... <configure
        ...   xmlns="http://namespaces.zope.org/zope"
        ...   xmlns:fss="http://namespaces.ingeniweb.com/filesystemstorage">
        ...   %s
        ... </configure>'''
        >>> zcml.load_config('meta.zcml', iw.fss)

    Existing product configuration::

        >>> atfile_directive = '''
        ... <fss:typeWithFSS
        ...   class="Products.ATContentTypes.atct.ATFile"
        ...   fields="file" />'''
        >>> config_zcml = template % atfile_directive
        >>> zcml.load_string(config_zcml)

    Make sure we configured it::

        >>> from Products.ATContentTypes.atct import ATFile
        >>> ATFile.schema['file'].storage
        <Storage FileSystemStorage>

    The patched type has been registered::

        >>> from iw.fss.zcml import patchedTypesRegistry
        >>> len(patchedTypesRegistry)
        1
        >>> patchedTypesRegistry[ATFile]
        {u'file': <Storage AnnotationStorage>}

    Not existing content type or class::

        >>> stupid_directive = '''
        ... <fss:typeWithFSS
        ...   class="no.such.class"
        ...   fields="woof" />'''
        >>> config_zcml = template % stupid_directive
        >>> zcml.load_string(config_zcml)
        Traceback (most recent call last):
        ...
        ZopeXMLConfigurationError: ...

    Class is not Archetypes content type::

        >>> stupid_directive = '''
        ... <fss:typeWithFSS
        ...   class="smtplib.SMTP"
        ...   fields="woof" />'''
        >>> config_zcml = template % stupid_directive
        >>> zcml.load_string(config_zcml)
        Traceback (most recent call last):
        ...
        ConfigurationExecutionError: exceptions.AttributeError: class SMTP has no attribute 'schema'
        ...

    No such field in valid content type::

        >>> stupid_directive = '''
        ... <fss:typeWithFSS
        ...   class="Products.ATContentTypes.atct.ATFile"
        ...   fields="file nosuchfield" />'''
        >>> config_zcml = template % stupid_directive
        >>> zcml.load_string(config_zcml)
        Traceback (most recent call last):
        ...
        ConfigurationExecutionError: exceptions.KeyError: u'nosuchfield'
        ...
    """

def test_suite():
    return unittest.TestSuite((
        doctest.DocTestSuite(optionflags=ELLIPSIS),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



## -*- coding: utf-8 -*-
## Copyright (C) 2006-2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Base test case
$Id: FSSTestCase.py 66391 2008-06-09 17:38:35Z glenfant $
"""

# Python imports
import os
import Globals

# Zope imports
import transaction
from Testing import ZopeTestCase
from AccessControl.SecurityManagement import newSecurityManager
from zope.component import getUtility

# CMF imports
from Products.CMFCore.utils import getToolByName

# Plone imports
from Products.PloneTestCase import PloneTestCase

# Products imports
from iw.fss.config import INSTALL_EXAMPLE_TYPES_ENVIRONMENT_VARIABLE
from iw.fss.interfaces import IConf

# Globals
portal_name = 'portal'
portal_owner = 'portal_owner'
default_user = PloneTestCase.default_user
default_password = PloneTestCase.default_password

STORAGE_PATH = os.path.join(Globals.INSTANCE_HOME, 'var', 'unittests_storage')
BACKUP_PATH = os.path.join(Globals.INSTANCE_HOME, 'var', 'unittests_backup')

DATA_PATH = os.path.join(os.path.dirname(__file__), 'data')
CONTENT_PATH = os.path.join(DATA_PATH, 'word.doc')
IMAGE_PATH = os.path.join(DATA_PATH, 'image.jpg')
CONTENT_TXT = """mytestfile"""


from Products.PloneTestCase.layer import onsetup
from Products.Five import fiveconfigure
from Products.Five import zcml
from Testing import ZopeTestCase as ztc

@onsetup
def setup_fss():

    fiveconfigure.debug_mode = True
    import iw.fss
    zcml.load_config('meta.zcml', iw.fss)
    zcml.load_config('configure.zcml', iw.fss)
    fiveconfigure.debug_mode = False
    ztc.installPackage('iw.fss')

setup_fss()

class FSSTestCase(PloneTestCase.PloneTestCase):

    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def _setup(self):
        PloneTestCase.PloneTestCase._setup(self)
        self.app.REQUEST['SESSION'] = self.Session()

        # Create temporary dirs to run test cases
        for base_path in (STORAGE_PATH, BACKUP_PATH):
            if not os.path.exists(base_path):
                os.mkdir(base_path)
        conf= getUtility(IConf, "globalconf")
        self.conf = conf()

        # Patch getStorageStragegy to test all strategies
        strategy_klass = self.strategy_klass
        def getStorageStrategy(self):
            return strategy_klass(STORAGE_PATH, BACKUP_PATH)

        from iw.fss.conffile import ConfFile
        ConfFile.getStorageStrategy = getStorageStrategy

        # Check if fss is switched
        self.use_atct = False
        ttool = getToolByName(self.portal, 'portal_types')
        info = ttool.getTypeInfo('Folder')
        if info.getProperty('meta_type') == 'ATFolder':
            self.use_atct = True

    def beforeTearDown(self):
        """Remove all the stuff again.
        """

        import shutil
        shutil.rmtree(STORAGE_PATH)
        shutil.rmtree(BACKUP_PATH)
        return

    def getDataPath(self):
        """Returns data path used for test cases"""

        return DATA_PATH

    def loginAsPortalOwner(self):
        '''Use if you need to manipulate an article as member.'''
        uf = self.app.acl_users
        user = uf.getUserById(portal_owner).__of__(uf)
        newSecurityManager(None, user)

    def addFileByString(self, folder, content_id):
        """Adds a file by string.
        """
        folder.invokeFactory('FSSItem', id=content_id)
        content = getattr(folder, content_id)
        transaction.savepoint(optimistic=True)
        kw = {'file' : CONTENT_TXT}
        content.edit(**kw)
        return content

    def addFileByFileUpload(self, folder, content_id):
        """Adds a file by file upload.
        """

        folder.invokeFactory('FSSItem', id=content_id)
        content = getattr(folder, content_id)
        transaction.savepoint(optimistic=True)
        self.updateContent(content, 'file', CONTENT_PATH)
        return content

    def addImageByFileUpload(self, folder, content_id):
        """
        Adding image
        """
        folder.invokeFactory('FSSItem', id=content_id)
        content = getattr(folder, content_id)
        transaction.savepoint(optimistic=True)
        self.updateContent(content, 'image', IMAGE_PATH)
        return content

    ## ATContentTypes based testcases

    def addATFileByString(self, folder, content_id):
        """Adds a file by string.
        """

        folder.invokeFactory('ATFSSItem', id=content_id)
        content = getattr(folder, content_id)
        transaction.savepoint(optimistic=True)
        kw = {'file' : CONTENT_TXT}
        content.edit(**kw)
        return content

    def addATFileByFileUpload(self, folder, content_id):
        """Adds a file by file upload.
        """

        folder.invokeFactory('ATFSSItem', id=content_id)
        content = getattr(folder, content_id)
        transaction.savepoint(optimistic=True)
        self.updateContent(content, 'file', CONTENT_PATH)
        return content

    def addATImageByFileUpload(self, folder, content_id):
        """
        Adding image
        """
        folder.invokeFactory('ATFSSItem', id=content_id)
        content = getattr(folder, content_id)
        transaction.savepoint(optimistic=True)
        self.updateContent(content, 'image', IMAGE_PATH)
        return content

    # Update content for any type

    def updateContent(self, content, field, filepath):
        """Updates a field content for a file.
        """

        from dummy import FileUpload
        file = open(filepath, 'rb')
        file.seek(0)
        filename = filepath.split('/')[-1]
        fu = FileUpload(filename=filename, file=file)
        kw = {field: fu}
        content.edit(**kw)

DEFAULT_PRODUCTS = ['kupu', 'iw.fss'] # 'FileSystemStorage']

# Install FSS Example types
os.environ[INSTALL_EXAMPLE_TYPES_ENVIRONMENT_VARIABLE] = 'True'

HAS_ATCT = True
ZopeTestCase.installProduct('ATContentTypes')

# Setup Plone site
PloneTestCase.setupPloneSite(products=DEFAULT_PRODUCTS, extension_profiles=['iw.fss:testfixtures'])




## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Stubs for testing
$Id: dummy.py 61129 2008-03-25 16:00:44Z glenfant $
"""

from ZPublisher.HTTPRequest import FileUpload as BaseFileUpload


class FileUpload(BaseFileUpload):
    """Dummy upload object.

    Used to fake uploaded files and images.
    """

    __allow_access_to_unprotected_subobjects__ = 1

    filename = 'dummy.gif'
    headers = {}

    def __init__(self, filename=None, headers=None, file=None):
        self.file = file
        if filename is not None:
            self.filename = filename
        if headers is not None:
            self.headers = headers

    def seek(self,*args):
        return self.file.seek(*args)

    def tell(self,*args):
        return self.file.tell(*args)

    def read(self,*args):
        return self.file.read(*args)



# -*- coding: utf-8 -*-
# Copyright (c) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Generic Test case for iw.fss doctests
"""
__docformat__ = 'restructuredtext'

import unittest
import doctest
import sys
import os

from Testing.ZopeTestCase import FunctionalDocFileSuite
from zope.publisher.browser import TestRequest
from Products.Five.testbrowser import Browser

from iw.fss import strategy
from iw.fss.conffile import ConfFile

from base import TestCase, STORAGE_PATH, BACKUP_PATH

current_dir = os.path.dirname(__file__)

def doc_suite(test_dir, setUp=None, tearDown=None, globs=None):
    """Returns a test suite, based on doctests found in /doctest."""    
    suite = []
    if globs is None:
        globs = globals()

    # preparing a few elements
    globs['test_dir'] = current_dir
    browser = Browser()
    browser.handleErrors = False
    globs['browser'] = browser

    flags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE |
             doctest.REPORT_ONLY_FIRST_FAILURE |
             doctest.REPORT_UDIFF)

    package_dir = os.path.split(test_dir)[0]
    if package_dir not in sys.path:
        sys.path.append(package_dir)

    docs = []
    for dir_ in ('doctests', ):
        doctest_dir = os.path.join(package_dir, dir_)

        # filtering files on extension
        docs.extend([os.path.join(doctest_dir, doc) for doc in
                     os.listdir(doctest_dir) if doc.endswith('.txt')])

    for test in docs:
        suite.append(FunctionalDocFileSuite(test, optionflags=flags,
                                            globs=globs, setUp=setUp,
                                            tearDown=tearDown,
                                            test_class=TestCase,
                                            module_relative=False))

    return suite


def test_suite():
    """returns the test suite"""
    
    strategies = ('FlatStorageStrategy', 'DirectoryStorageStrategy',
                  'SiteStorageStrategy', 'SiteStorageStrategy2')
    
    def changeStrategy(self):
        strategy_klass = self.globs['strategy_klass'](STORAGE_PATH, BACKUP_PATH)
        ConfFile.getStorageStrategy = lambda x: strategy_klass
        
    # Duplicate doc tests for all strategies
    suite = []
    for strategy_name in strategies:
        strategy_klass = getattr(strategy, strategy_name)

        suite.extend(doc_suite(current_dir, globs={'strategy_klass': strategy_klass},
                                            setUp=changeStrategy))
    
    return unittest.TestSuite(suite)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
from Globals import InitializeClass

from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.utils import getToolByName

from Products.CMFEditions.interfaces.IModifier import ISaveRetrieveModifier
from Products.CMFEditions.interfaces.IArchivist import ArchivistUnregisteredError
from Products.CMFEditions.Modifiers import ConditionalTalesModifier

from iw.fss.FileSystemStorage import FileSystemStorage

MODIFIER_ID = 'FSSModifier'

modifierAddForm = PageTemplateFile('zmi/modifier_add_form.zpt',
    globals(), __name__='modifier_add_form')

def manage_addModifier(self, REQUEST=None):
    """Add a modifier for saving versions in FileSystemStorage.
    """

    # Add modifier
    modifier = Modifier()
    id = MODIFIER_ID
    title = "A modifier for saving FileSystemStorage versions"
    wrapped = ConditionalTalesModifier(id, modifier, title)
    wrapped.edit(enabled=True, condition='python:True', title=title)
    self.register(id, wrapped)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')


class Modifier(object):
    """
   Save each version of an FSS file into a different file so past versions of
   files are available
    """

    __implements__ = (ISaveRetrieveModifier, )

    def beforeSaveModifier(self, obj, clone):
        for field in obj.Schema().fields():
            if not hasattr(field, "getStorage"):
                # For a field that doesn't use storage
                continue

            storage = field.getStorage()

            if not isinstance(storage, FileSystemStorage):
                continue

            rtool = getToolByName(obj, 'portal_repository')
            history = rtool.getHistory(obj)
            version = len(history)

            storage.set(
                field.getName(),
                clone.__of__(obj.aq_parent),
                field.getAccessor(obj)(),
                mimetype=field.getContentType(obj),
                field=field,
                filename=field.getFilename(obj),
                version=version)

        return {}, [], []

    def afterRetrieveModifier(self, obj, repo_clone, preserve=()):
        return [], [], {}


## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
$Id: utils.py 65108 2008-05-19 20:23:31Z cbosse $
"""

__author__  = ''
__docformat__ = 'restructuredtext'

import os
import types

from AccessControl import ModuleSecurityInfo
from zope.i18nmessageid import MessageFactory

# Archetypes imports
from Products.Archetypes.Field import ImageField
#from FileSystemStorage import FileSystemStorage

import config

def getFieldValue(self, name):
    """Returns field value of an object

    @param name: Name of the field
    """

    field = self.getField(name)
    if not field:
        # Get Image fields
        # Check for scales
        found = False
        fields = [x for x in self.Schema().fields() if isinstance(x, ImageField)]
        for field in fields:
            field_name = field.getName()
            names = ['%s_%s' % (field_name, x) for x in field.getAvailableSizes(self).keys()]
            if name in names:
                obj = field.getStorage(self).get(name, self)
                found = True
                break
        if not found:
            raise AttributeError(name)
        return obj.__of__(self)

    # Standard field
    accessor = field.getAccessor(self)

    if accessor is None:
        return None

    return accessor()


FSSMessageFactory = MessageFactory(config.I18N_DOMAIN)
ModuleSecurityInfo('iw.fss.utils').declarePublic('FSSMessageFactory')

###
## File management utilities
###

def rm_file(path):
    """
    Delete file from a path
    """

    os.remove(path)

def move_file(infile, outfile):
    """
    Move file to another place
    """

    if infile == outfile:
        # SiteStorageStrategy can send equals infile and outfile
        return

    if os.path.exists(outfile) and infile != outfile:
        os.remove(outfile)

    os.rename(infile, outfile)


def copy_file(infile, outfile):
    """

    Read binary data from infile and write it to outfile. infile and
    outfile my be strings, in which case a file with that name is
    opened, or filehandles, in which case they are accessed directly.

    """

    if type(infile) is types.StringType:
        instream = open(infile, 'rb')
        close_in = 1
    else:
        close_in = 0
        instream = infile

    if type(outfile) is types.StringType:
        outstream = open(outfile, 'wb')
        close_out = 1
    else:
        close_out = 0
        outstream = outfile
    try:
        blocksize = 2<<16
        block = instream.read(blocksize)
        outstream.write(block)
        while len(block)==blocksize:
            block = instream.read(blocksize)
            outstream.write(block)
        instream.seek(0)
    finally:
        if close_in: instream.close()
        if close_out: outstream.close()

def create_file(contents_file, target_filepath):
    """
    Creates a physical file on the filesystem at target_filepath.  If
    contents_file is not None, it points to a file whose contents will
    be copied into the target file.  Otherwise, a zero length file
    will be created.  This method will raise an exception if there are
    any problems writing to the target file (or reading
    contents_file).
    """

    if contents_file:
        copy_file(contents_file, target_filepath)
    else:
        touchfile = open(target_filepath, 'wb') # create zero-length file
        touchfile.close()



# -*- coding: utf-8 -*-
## Copyright (C) 2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
FSS configuration schema factory
$Id: schema.py 63747 2008-04-27 13:46:09Z glenfant $
"""

__author__  = 'Gilles Lenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

import os

from ZConfig.datatypes import Registry
from ZConfig.loader import SchemaLoader

import datatypes

_this_dir = os.path.dirname(os.path.abspath(__file__))
SCHEMA_FILE = os.path.join(_this_dir, "schema.xml")

# Our registry
fssRegistry = Registry(stock=None)
fssRegistry.register('existing-storage-path', datatypes.existingStoragePath)
fssRegistry.register('existing-backup-path', datatypes.existingBackupPath)
fssRegistry.register('default-strategy', datatypes.default_strategy)
fssRegistry.register('strategy', datatypes.strategy)

# Our configuration schema
fssSchema = None
def loadSchema(filepath, registry=fssRegistry, overwrite=False):
    """Sets up fssSchema
    @param filepath: path to schema xml file
    @param registry: ZConfig.datatypes.Registry
    @param overwrite: True to change fss
    """

    global fssSchema
    if fssSchema is not None and not overwrite:
        raise RuntimeError, 'Schema already loaded'
    schemaLoader = SchemaLoader(registry=registry)
    fssSchema = schemaLoader.loadURL(filepath)
    return fssSchema

loadSchema(SCHEMA_FILE)

__all__ = ('fssSchema')



## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from Products.CMFCore.utils import getToolByName

from iw.fss.modifier import MODIFIER_ID
from iw.fss.modifier import manage_addModifier

def thisProfileOnly(func):
    """Decorator that prevents the setup func to be used on other GS profiles.
    Usage:
    @thisProfileOnly
    def someFunc(context): ...
    """

    def wrapper(context):
        if context.readDataFile('iw.fss.txt') is None:
            return
        else:
            return func(context)
    return wrapper

@thisProfileOnly
def setupVarious(context):
    """Put here various stuff that cannot be installed with generic setup"""

    portal = context.getSite()

    # Install CMFEditions modifier for FileSystemStorage
    mtool = getToolByName(portal, 'portal_modifier')

    if not MODIFIER_ID in  mtool.objectIds():
        manage_addModifier(mtool)
